@@ -12,12 +12,13 @@ import numpy as np
 import numpy.linalg as la
 from . import ols as OLS
 from . import user_output as USER
-from . import summary_output as SUMMARY
 from . import twosls as TSLS
 from . import utils as UTILS
-from .utils import RegressionPropsY, spdot, set_endog, sphstack, set_warn
+from .utils import RegressionPropsY, spdot, set_endog, sphstack, set_warn, get_lags
 from scipy import sparse as SP
 from libpysal.weights.spatial_lag import lag_spatial
+import pandas as pd
+from .output import output, _summary_iteration, _spat_pseudo_r2
 
 __all__ = ["GM_Error_Het", "GM_Endog_Error_Het", "GM_Combo_Het"]
 
@@ -170,6 +171,9 @@ class GM_Error_Het(BaseGM_Error_Het):
                    independent (exogenous) variable, excluding the constant
     w            : pysal W object
                    Spatial weights object
+    slx_lags     : integer
+                   Number of spatial lags of X to include in the model specification.
+                   If slx_lags>0, the specification becomes of the SDEM type.
     max_iter     : int
                    Maximum number of iterations of steps 2a and 2b from :cite:`Arraiz2010`.
                    Note: epsilon provides an additional
@@ -191,9 +195,13 @@ class GM_Error_Het(BaseGM_Error_Het):
                    Name of weights matrix for use in output
     name_ds      : string
                    Name of dataset for use in output
+    latex        : boolean
+                   Specifies if summary is to be printed in latex format
 
     Attributes
     ----------
+    output       : dataframe
+                   regression results pandas dataframe
     summary      : string
                    Summary of regression results and diagnostics (note: use in
                    conjunction with the print command)
@@ -336,6 +344,7 @@ class GM_Error_Het(BaseGM_Error_Het):
         y,
         x,
         w,
+        slx_lags=0,
         max_iter=1,
         epsilon=0.00001,
         step1c=False,
@@ -344,6 +353,7 @@ class GM_Error_Het(BaseGM_Error_Het):
         name_x=None,
         name_w=None,
         name_ds=None,
+        latex=False,
     ):
 
         n = USER.check_arrays(y, x)
@@ -351,6 +361,12 @@ class GM_Error_Het(BaseGM_Error_Het):
         USER.check_weights(w, y, w_required=True)
         x_constant, name_x, warn = USER.check_constant(x, name_x)
         set_warn(self, warn)
+        self.title = "GM SPATIALLY WEIGHTED LEAST SQUARES (HET)"
+        if slx_lags >0:
+            lag_x = get_lags(w, x_constant[:, 1:], slx_lags)
+            x_constant = np.hstack((x_constant, lag_x))
+            name_x += USER.set_name_spatial_lags(name_x, slx_lags)
+            self.title += " WITH SLX (SDEM)"
         BaseGM_Error_Het.__init__(
             self,
             y,
@@ -360,13 +376,16 @@ class GM_Error_Het(BaseGM_Error_Het):
             step1c=step1c,
             epsilon=epsilon,
         )
-        self.title = "SPATIALLY WEIGHTED LEAST SQUARES (HET)"
         self.name_ds = USER.set_name_ds(name_ds)
         self.name_y = USER.set_name_y(name_y)
         self.name_x = USER.set_name_x(name_x, x_constant)
         self.name_x.append("lambda")
         self.name_w = USER.set_name_w(name_w, w)
-        SUMMARY.GM_Error_Het(reg=self, w=w, vm=vm)
+        self.output = pd.DataFrame(self.name_x, columns=['var_names'])
+        self.output['var_type'] = ['x'] * (len(self.name_x)-1) + ['lambda']
+        self.output['regime'], self.output['equation'] = (0, 0)
+        self.other_top = _summary_iteration(self)
+        output(reg=self, vm=vm, robust=False, other_end=False, latex=latex)
 
 
 class BaseGM_Endog_Error_Het(RegressionPropsY):
@@ -582,6 +601,9 @@ class GM_Endog_Error_Het(BaseGM_Endog_Error_Het):
                    this should not contain any variables from x)
     w            : pysal W object
                    Spatial weights object
+    slx_lags     : integer
+                   Number of spatial lags of X to include in the model specification.
+                   If slx_lags>0, the specification becomes of the SDEM type.
     max_iter     : int
                    Maximum number of iterations of steps 2a and 2b from
                    :cite:`Arraiz2010`. Note: epsilon provides an additional
@@ -611,9 +633,13 @@ class GM_Endog_Error_Het(BaseGM_Endog_Error_Het):
                    Name of weights matrix for use in output
     name_ds      : string
                    Name of dataset for use in output
+    latex        : boolean
+                   Specifies if summary is to be printed in latex format
 
     Attributes
     ----------
+    output       : dataframe
+                   regression results pandas dataframe
     summary      : string
                    Summary of regression results and diagnostics (note: use in
                    conjunction with the print command)
@@ -788,6 +814,7 @@ class GM_Endog_Error_Het(BaseGM_Endog_Error_Het):
         yend,
         q,
         w,
+        slx_lags=0,
         max_iter=1,
         epsilon=0.00001,
         step1c=False,
@@ -799,6 +826,7 @@ class GM_Endog_Error_Het(BaseGM_Endog_Error_Het):
         name_q=None,
         name_w=None,
         name_ds=None,
+        latex=False,
     ):
 
         n = USER.check_arrays(y, x, yend, q)
@@ -806,6 +834,12 @@ class GM_Endog_Error_Het(BaseGM_Endog_Error_Het):
         USER.check_weights(w, y, w_required=True)
         x_constant, name_x, warn = USER.check_constant(x, name_x)
         set_warn(self, warn)
+        self.title = "GM SPATIALLY WEIGHTED TWO STAGE LEAST SQUARES (HET)"
+        if slx_lags > 0:
+            lag_x = get_lags(w, x_constant[:, 1:], slx_lags)
+            x_constant = np.hstack((x_constant, lag_x))
+            name_x += USER.set_name_spatial_lags(name_x, slx_lags)
+            self.title += " WITH SLX (SDEM)"
         BaseGM_Endog_Error_Het.__init__(
             self,
             y=y,
@@ -818,7 +852,6 @@ class GM_Endog_Error_Het(BaseGM_Endog_Error_Het):
             epsilon=epsilon,
             inv_method=inv_method,
         )
-        self.title = "SPATIALLY WEIGHTED TWO STAGE LEAST SQUARES (HET)"
         self.name_ds = USER.set_name_ds(name_ds)
         self.name_y = USER.set_name_y(name_y)
         self.name_x = USER.set_name_x(name_x, x_constant)
@@ -828,7 +861,12 @@ class GM_Endog_Error_Het(BaseGM_Endog_Error_Het):
         self.name_q = USER.set_name_q(name_q, q)
         self.name_h = USER.set_name_h(self.name_x, self.name_q)
         self.name_w = USER.set_name_w(name_w, w)
-        SUMMARY.GM_Endog_Error_Het(reg=self, w=w, vm=vm)
+        self.output = pd.DataFrame(self.name_z,
+                                   columns=['var_names'])
+        self.output['var_type'] = ['x'] * len(self.name_x) + ['yend'] * len(self.name_yend) + ['lambda']
+        self.output['regime'], self.output['equation'] = (0, 0)
+        self.other_top = _summary_iteration(self)
+        output(reg=self, vm=vm, robust=False, other_end=False, latex=latex)
 
 
 class BaseGM_Combo_Het(BaseGM_Endog_Error_Het):
@@ -1026,6 +1064,10 @@ class GM_Combo_Het(BaseGM_Combo_Het):
                    Orders of W to include as instruments for the spatially
                    lagged dependent variable. For example, w_lags=1, then
                    instruments are WX; if w_lags=2, then WX, WWX; and so on.
+    slx_lags     : integer
+                   Number of spatial lags of X to include in the model specification.
+                   If slx_lags>0, the specification becomes of the General Nesting
+                   Spatial Model (GNSM) type.
     lag_q        : boolean
                    If True, then include spatial lags of the additional
                    instruments (q).
@@ -1058,9 +1100,13 @@ class GM_Combo_Het(BaseGM_Combo_Het):
                    Name of weights matrix for use in output
     name_ds      : string
                    Name of dataset for use in output
+    latex        : boolean
+                   Specifies if summary is to be printed in latex format
 
     Attributes
     ----------
+    output       : dataframe
+                   regression results pandas dataframe
     summary      : string
                    Summary of regression results and diagnostics (note: use in
                    conjunction with the print command)
@@ -1255,6 +1301,7 @@ class GM_Combo_Het(BaseGM_Combo_Het):
         q=None,
         w=None,
         w_lags=1,
+        slx_lags=0,
         lag_q=True,
         max_iter=1,
         epsilon=0.00001,
@@ -1267,6 +1314,7 @@ class GM_Combo_Het(BaseGM_Combo_Het):
         name_q=None,
         name_w=None,
         name_ds=None,
+        latex=False,
     ):
 
         n = USER.check_arrays(y, x, yend, q)
@@ -1274,7 +1322,13 @@ class GM_Combo_Het(BaseGM_Combo_Het):
         USER.check_weights(w, y, w_required=True)
         x_constant, name_x, warn = USER.check_constant(x, name_x)
         set_warn(self, warn)
-        yend2, q2 = set_endog(y, x_constant[:, 1:], w, yend, q, w_lags, lag_q)
+
+        if slx_lags == 0:
+            yend2, q2 = set_endog(y, x_constant[:, 1:], w, yend, q, w_lags, lag_q)
+        else:
+            yend2, q2, wx = set_endog(y, x_constant[:, 1:], w, yend, q, w_lags, lag_q, slx_lags)
+            x_constant = np.hstack((x_constant, wx))
+
         BaseGM_Combo_Het.__init__(
             self,
             y=y,
@@ -1294,7 +1348,10 @@ class GM_Combo_Het(BaseGM_Combo_Het):
             w, self.y, self.predy, yend2[:, -1].reshape(self.n, 1), self.rho
         )
         UTILS.set_warn(self, warn)
-        self.title = "SPATIALLY WEIGHTED TWO STAGE LEAST SQUARES (HET)"
+        self.title = "SPATIALLY WEIGHTED 2SLS- GM-COMBO MODEL (HET)"
+        if slx_lags > 0:
+            name_x += USER.set_name_spatial_lags(name_x, slx_lags)
+            self.title += " WITH SLX (GNSM)"
         self.name_ds = USER.set_name_ds(name_ds)
         self.name_y = USER.set_name_y(name_y)
         self.name_x = USER.set_name_x(name_x, x_constant)
@@ -1306,7 +1363,13 @@ class GM_Combo_Het(BaseGM_Combo_Het):
         self.name_q.extend(USER.set_name_q_sp(self.name_x, w_lags, self.name_q, lag_q))
         self.name_h = USER.set_name_h(self.name_x, self.name_q)
         self.name_w = USER.set_name_w(name_w, w)
-        SUMMARY.GM_Combo_Het(reg=self, w=w, vm=vm)
+        self.output = pd.DataFrame(self.name_z,
+                                   columns=['var_names'])
+        self.output['var_type'] = ['x'] * len(self.name_x) + ['yend'] * (len(self.name_yend)-1) + ['rho', 'lambda']
+        self.output['regime'], self.output['equation'] = (0, 0)
+        self.other_top = _spat_pseudo_r2(self)
+        self.other_top += _summary_iteration(self)
+        output(reg=self, vm=vm, robust=False, other_end=False, latex=latex)
 
 
 # Functions
@@ -1551,3 +1614,28 @@ def _test():
 
 if __name__ == "__main__":
     _test()
+    import numpy as np
+    import libpysal
+
+    db = libpysal.io.open(libpysal.examples.get_path('columbus.dbf'),'r')
+    y = np.array(db.by_col("HOVAL"))
+    y = np.reshape(y, (49,1))
+    X = []
+    X.append(db.by_col("INC"))
+    X = np.array(X).T
+    yd = []
+    yd.append(db.by_col("CRIME"))
+    yd = np.array(yd).T
+    q = []
+    q.append(db.by_col("DISCBD"))
+    q = np.array(q).T
+
+    w = libpysal.weights.Rook.from_shapefile(libpysal.examples.get_path("columbus.shp"))
+    w.transform = 'r'
+    # reg = GM_Error_Het(y, X, w=w, name_x=['inc'], name_y='hoval', name_ds='columbus', vm=True)
+    # reg = GM_Endog_Error_Het(y, X, yd, q, w=w, name_x=['inc'], name_y='hoval', name_yend=['crime'],
+    #                         name_q=['discbd'], name_ds='columbus',vm=True)
+    reg = GM_Combo_Het(y, X, yd, q, w=w, step1c=True, name_x=['inc'], name_y='hoval', name_yend=['crime'], name_q=['discbd'], name_ds='columbus',
+                       vm=True)
+    print(reg.output)
+    print(reg.summary)
\ No newline at end of file
